function ChromeSystemController(){
    if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
	}
	arguments.callee._singletonInstance = this;

	var self = this; self.name = "ChromeSystemController"; // MANDATORY
    this.rootDir = "/";
    this.appDir = "";
    this.ready_callback = false;
    //this.mainDirName = "ADMIRA/";

    this.fs = false;

    this.systemIsUpdating = false;


    this.dataInfoSystem = null;
    this.sendInfoSystem = false;
    this.superEco = false;
    this.eco = false;
    this.screenshotServer = null;
    this.id_content_now = null;

    this.busy = false;
    this.SocketConnector = undefined;

    this.sync_master_supported = false;

    this.init = function(callback){
      self.SocketConnector = new SocketConnector(self);
        self.ready_callback = callback;
        navigator.webkitPersistentStorage.requestQuota(0,self.onQuotaGranted,self.onQuotaError);
    };

    this.onQuotaGranted = function(grantedBytes){
        self.log("Granted Bytes : "+grantedBytes);
        self.log("checking folders .... asked");
        window.requestFileSystem  = window.requestFileSystem || window.webkitRequestFileSystem;
        window.requestFileSystem(1, grantedBytes, self.onInitFs, self.onErrorFsInit);

    };

    this.onQuotaError = function(e){
        console.log("Quota error ",e);
    };

    this.createTrayIcon = function(){

    };

    this.checkFolders = function(){
        var allExist = true;
        for(var route in self.ROUTES){
            if(self.ROUTES[route].isFolder && !self.ROUTES[route].exists){
                allExist = false;
                self.makeDir(self.ROUTES[route].path,function(){
                   self.ROUTES[route].exists = true;
                });
                break;
            }
        }
        if(allExist){
            self.checkCoreFile();
        }else{
            setTimeout(self.checkFolders,250);
        }
    };

    this.checkCoreFile = function(){
      self.createCoreFile();
        self.fileExists(self.ROUTES['CORE'].path,function(exists,file_ref){
            if(exists){
                self.checkConfigFile();
            }else{
                self.createCoreFile();
            }
        });
    };

    this.createCoreFile = function(){
        var xml_data = '<?xml version="1.0" encoding="utf-8" ?><core><config><id>0</id><api_ver>3.0</api_ver><api_url>apilb.admira.com</api_url></config><server url="apilb.admira.com"/></core>';
        self.writeFile(self.ROUTES['CORE'].path,xml_data,function(){
            self.checkConfigFile();
        });
        var style_data = '<?xml version="1.0" encoding="utf-8"?><!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg  xmlns="http://www.w3.org/2000/svg"  version="1.1" id="SVG_1"  x="0px" y="0px" enable-background="new 0 0 100% 100%" style="background-color:#000"><circle cx="98%" cy="98%" r="0.5%" style="stroke: #68983F; fill: #68983F;"><animate id="animation1" attributeName="opacity" from="0" to="1" dur="1s" begin="0s;animation2.end" /><animate id="animation2" attributeName="opacity" from="1" to="0" dur="1s" begin="animation1.end" /></circle></svg>';
        self.writeFile(self.ROUTES['STYLE'].path+"logo.svg",style_data,function(){
           self.log("LOGO File created ! ");
        });
    };

    this.checkConfigFile = function(){
        self.fileExists(self.ROUTES['CONFIG'].path,function(exists,file_ref){
            if(exists){
                self.checkDataXmlFile();
            }else{
                self.createConfigFile();
            }
        });
    };

    this.createConfigFile = function(){
      console.log("create config file");
        var xml_data = '<config><data file="datos.xml"/><paths multiplayer_delimiter="_" xmls="xmls/" content="content/" temp="temp/" modules="modules/" log="log/" stadistics="stats/"/><params bumper_content="assets/downloading.swf" bumper_duration="56" alwaysontop="1"/></config>';
        self.writeFile(self.ROUTES['CONFIG'].path,xml_data,function(){
            self.checkDataXmlFile();
        });
    };

    this.checkDataXmlFile = function(){
        var filename = self.getDayOfWeek()+".xml";
        self.fileExists(self.ROUTES['XML'].path+filename,function(exists,file_ref){
            if(exists){
                self.ready_callback();
            }else{
                self.createDataXmlFile();
            }
        });
    };

    this.createDataXmlFile = function(){
       console.log(" Creating data file");
        var filename = self.getDayOfWeek()+".xml";
        var xml_data = '<?xml version="1.0"?><admira timestamp="1" version="1.0" status="ok"><archive/><playlist><loop/></playlist><screens /><config volume="100" fullscreen="true" mode="1" always_on_top="true" plv_name="Debug Name" project_name="Debug Project" time="30" plv_id="0" quit_on_click="true" activated="false" domain="" api_key="" live_url="" live_recheck_seconds="" live_send_id=""><resolution x="100" y="100" width="300" height="300" /></config></admira>';
        self.writeFile(self.ROUTES['XML'].path+filename,xml_data,function(){
            console.log(self.ROUTES['XML'].path+filename+" Created ....");
            self.ready_callback();
        });
    }

    this.onInitFs = function(fs){
        self.fs = fs;
        setTimeout(self.checkFolders,250);
    };

    this.onErrorFsInit = function(e){
        console.log("Filesystem error ",e);
    };

    this.onResolveError = function(e){
        console.log("Resolve Path error ",e);
    };

    this.onFileError = function(e){
        // https://developer.mozilla.org/en-US/docs/Web/API/FileError
        var msg = '';
        switch (e.code) {
            case FileError.QUOTA_EXCEEDED_ERR:
                msg = 'Storage quota exceeded';
                break;
            case FileError.NOT_FOUND_ERR:
                msg = 'File not found';
                break;
            case FileError.SECURITY_ERR:
                msg = 'Security error';
                break;
            case FileError.INVALID_MODIFICATION_ERR:
                msg = 'Invalid modification';
                break;
            case FileError.INVALID_STATE_ERR:
                msg = 'Invalid state';
                break;
            default:
                msg = 'Unknown error';
                break;
        };
        console.log("File ERROR : "+msg);
    };

    this.writeFile = function(filename,value,callback,params){
            self.fs.root.getFile(filename,{create: true},function(fileEntry){
                fileEntry.createWriter(function (fileWriter) {
                    fileWriter.onwriteend = function (e) {
                        if (fileWriter.length === 0) {
                            var blob = new Blob([value]);
                            fileWriter.write(blob);
                            if(typeof(callback) !== "undefined"){
                                callback(true,filename,params);
                            }
                        }
                    };
                    fileWriter.onerror = function(e){
                        if(typeof(callback) !== "undefined"){
                            callback(false,filename,params);
                        }
                    };
                    fileWriter.truncate(0);
                }, self.onFileError.bind(null, filename));
            }, self.onFileError.bind(null, filename));
    };

    this.appendFile = function(filename,value,callback,params){
        self.fs.root.getFile(filename,{create: true},function(fileEntry){
            fileEntry.createWriter(function (fileWriter) {
                fileWriter.seek(fileWriter.length);
                fileWriter.onwriteend = function (e) {
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params);
                    }
                };
                fileWriter.onerror = function(e){
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                };
                var blob = new Blob([value]);
                fileWriter.write(blob);
                //fileWriter.truncate(0);
            }, self.onFileError.bind(null, filename));
        }, self.onFileError.bind(null, filename));
    };

    this.writeBlob = function(filename,blob,callback,params){
            self.fs.root.getFile(filename,{create: true},function(fileEntry){
                fileEntry.createWriter(function (fileWriter) {
                    fileWriter.onwriteend = function (e) {
                        if (fileWriter.length === 0) {
                            fileWriter.write(blob);
                            if(typeof(callback) !== "undefined"){
                                callback(true,filename,params);
                            }
                        }
                    };
                    fileWriter.onerror = function(e){
                        console.log("BLOB Write File "+filename+" Error : "+e.toString());
                        if(typeof(callback) !== "undefined"){
                            callback(false,filename,params);
                        }
                    };
                    fileWriter.truncate(0);
                }, self.onFileError.bind(null, filename));
            }, self.onFileError.bind(null, filename));
    };

    this.readFile = function(filename,callback,params){
        var pathToFile =  filename;
        self.fs.root.getFile(pathToFile,{},function(fileEntry){
            fileEntry.file(function(file){
                var reader = new FileReader();
                reader.onloadend = function (e) {
                    var value = this.result;
                    if(typeof(callback) !== "undefined"){
                        callback(value,filename,params);
                    }
                };
                reader.onerror = function (e) {
                    if(typeof(callback) !== "undefined"){
                        callback("",filename,e);
                    }
                };
                reader.readAsText(file);
            }, function(e){
                console.log("File read "+filename+" Error : "+e);
                if(typeof(callback) !== "undefined"){
                    callback("",filename,e);
                }
            });
        },function(e){
            console.log("File resolve "+filename+" Error ");
            console.log(e);
            if(typeof(callback) !== "undefined"){
                callback("",filename,e);
            }
        });
    };

    this.fileExists = function(filename,callback,params){
      self.fs.root.getFile(self.rootDir+filename,{create:false}, function (fileEntry) {
          if(typeof(callback) !== "undefined"){
              callback(true,fileEntry,params);
          }
      }, function(e){
          if(typeof(callback) !== "undefined"){
              callback(false,filename,params);
          }
      });
    };

    this.makeDir = function(path,callback,params){
      self.fs.root.getDirectory(self.rootDir+path,{create:true, exclusive:false}, function (directoryEntry) {
          if(typeof(callback) !== "undefined"){
              callback(true,path,params);
          }
      }, function(e){
          console.log(e);
          if(typeof(callback) !== "undefined"){
              callback(false,path,params);
          }
      });
    };

    this.modificationDate = function(filename,callback,params){
        self.fs.root.getFile(self.rootDir+filename,{}, function (fileEntry) {
            fileEntry.getMetadata(function(metadata){
                var date = new Date(metadata.modificationTime).getTime();
                if(typeof(callback) !== "undefined"){
                    callback(date,filename,params);
                }
            }, function(error){
                console.log("Checking Modification ERROR ");
                if(typeof(callback) !== "undefined"){
                    callback(-1,filename,params);
                }
            });
        }, self.onResolveError.bind(null, filename));
    };

    this.downloadFile = function(url,destFolder,callback,params){
        var filename = url;
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);
        if(typeof(params) !== "undefined"){
            if(typeof(params.filename) !== "undefined"){
                filename = params.filename;
            }
        }
        var uri = encodeURI(url);
        var xhr = new XMLHttpRequest();
        xhr.overrideMimeType("application/octet-stream");
        xhr.open("GET", uri, true);
        xhr.setRequestHeader('Access-Control-Allow-Origin', '*');
        xhr.responseType = "arraybuffer";
        xhr.onload = function() {
              var res = xhr.response;
              if (res){
                  var byteArray = new Uint8Array(res);

              }

              var bb = new Blob([byteArray.buffer],{type:"application/octet-stream"});//(window.BlobBuilder || window.WebKitBlobBuilder)();
              self.writeBlob(self.rootDir+destFolder+filename,bb,function(success){
                if(typeof(callback) !== "undefined"){
                        callback(success,url,params);
                }
              });
        };
        xhr.send(null);

    };

    this.unzipFile = function(filename,destFolder,callback,params){

        console.log("Trying to unzip file : "+filename+" to "+destFolder);
        JSZipUtils.getBinaryContent(self.fs.root.toURL()+filename, function(err, data) {
            if(err) {
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
                throw err; // or handle err
            }

            var zip = new JSZip(data);
            for(var file in zip.files){
                self.log("Unzip file item : "+file);
                self.writeFile(self.ROUTES['XML'].path+file,zip.files[file].asText());
            }
            if(typeof(callback) !== "undefined"){
                callback(true,file,params);
            }
        });

        if(typeof(callback) !== "undefined"){
                callback(true,filename,params);
            }
    };

    this.readDirectory = function(directory,callback,params){
        self.fs.root.getDirectory(self.rootDir+directory,{create:false, exclusive:false},function(directoryEntry){
            var reader = directoryEntry.createReader();
            var files = [];
            reader.readEntries (function(fileEntries) {
                for(var i=0;i<fileEntries.length;i++){
                    files.push(fileEntries[i].name);
                }
                if(typeof(callback) !== "undefined"){
                    callback(files,directory,params);
                }
            },function(e){
                console.log(e);
                if(typeof(callback) !== "undefined"){
                    callback(files,directory,params);
                }
            });
        }, self.onResolveError.bind(null, directory));
    };

    this.watchDirectory = function(directory,compare_files,callback,params){
        // TODO

    };

    this.deleteFile = function(filename,callback,params){
        self.fs.root.getDirectory(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create:false,exclusive:false}, function (fileEntry) {
                fileEntry.remove(function(entry){
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params);
                    }
                },function(e){
                    console.log("Error deleting file ... ");
                    console.log(e);
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                });
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
            });
        }, self.onResolveError.bind(null, filename));
    };

    this.deleteFilesInDir = function(dir,filter,callback,params){
      // TODO
    };

    this.moveFile = function(source,destination,callback,params){
        if(typeof(callback) !== "undefined"){
            callback(false,"",params);
        }
    };

    this.systemUpdate = function(update_folder,platform){
        // TODO
    };

    this.systemRepairUpdate = function(){
      // TODO
    };

    this.takeScreenshot = function(callback,params){
        /*
        navigator.screenshot.save(function(error,res){
            if(error){
                console.error(error);
                if(typeof(callback) !== "undefined"){
                    callback(false,params);
                }
            }else{
                ok = true;
                var img = new Image();
                img.crossOrigin = 'Anonymous';
                img.onload = function(){
                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                    if(typeof(callback) !== "undefined"){
                        callback(base64.substring(23),params);
                    }
                    canvas = null;
                };
                var d = new Date();
                img.src = res.filePath+"?time="+d;
            }
        },'jpg',75,"capture");
        */
        /*
        try{
          console.log(chrome.tabs);

          chrome.tabs.captureVisibleTab(null, {}, function (image) {
   // You can add that image HTML5 canvas, or Element.

                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                    if(typeof(callback) !== "undefined"){
                      console.log(base64.substring(23));
                        callback(base64.substring(23),params);
                    }
                    canvas = null;

});
        }catch(e){
          console.log(e);
        }
  */

            /*
        chrome.runtime.sendMessage("hola", function(img) {
            console.log(img);
                                var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                    canvas = null;
            console.log("Runtime Response : "+base64);
        });
                    */

         if(typeof(callback) !== "undefined"){
            callback("",params);
        }
    };

    this.setSystemDateTime = function(timestamp,callback,params){
        // TODO
    };

    this.rebootSystem = function(platform){
        // TODO
    };

    this.haltSystem = function(){
         // TODO
    };

    this.internetConnected = function() {
        // TODO
        return true;
    };

    this.getNetworkInfo = function(callback,params){
        // TODO
        var network_info = {
            ip: "0.0.0.0",
            netmask : "0.0.0.0",
            gateway : "0.0.0.0"
        };
        return network_info;
    };

    this.getScreens = function(callback){
        // TODO Use Callback
        var screens = [];
        screens[0] = {id:0,bounds:{x:0,y:0,width:400,height:300,scaleFactor:1}};
        chrome.runtime.sendMessage({action:"getScreens"}, function (appScreens){
            console.log("Chrome API : Get Screens Info ");
            console.log(appScreens[0].bounds);
            screens = appScreens;
            if(typeof(callback) !== "undefined"){
              callback(appScreens);
            }

        });
        return screens;
    };

    this.setWindow = function(x,y,w,h){
        // OVERRIDE
    };

    this.setProxyConfig = function(config){
        // OVERRIDE
    };

    this.getProxyForURL = function(url){
        // OVERRIDE
        return "";
    };

    this.getAddWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getRemoveWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getDataPath = function(){
        return self.rootDir;
    };

    this.setCrashDumpPath = function(folderPath){
         // OVERRIDE
    };

    this.quit = function(){
      chrome.runtime.sendMessage({action:"close"}, function(val) {
            console.log("Chrome API : Close Window ");
        });
    };

    this.clearCache = function(){
        // OVERRIDE
    };

    this.alwaysOnTop = function(value){
        chrome.runtime.sendMessage({action:"setAlwaysOnTop",value:value}, function(val) {
            console.log("Chrome API : Always on top set");
        });
    };

    this.focusWindow = function(){
       chrome.runtime.sendMessage({action:"focus"}, function(val) {
            console.log("Chrome API : Focus Window ");
        });
    };

    this.disableRemoteControl = function(platform){
        // TODO
    };

    this.enableRemoteControl = function(platform,tunnels){
        // TODO
    };

    this.keepAwake = function(platform){
        // TODO : Keeps Screen Awake
    };

    this.socketServerStart = function(port,onOpenCallback,onMessageCallback,onTerminatedCallbackk){
      // TODO : NOT AVAILABLE
    };

    this.socketServerBroadcast = function(msg,exclude){
      // TODO : NOT AVAILABLE
    };


    this.activateControl = function(){
      self.debug = true;
      self.SocketConnector.createSocket();
    };
    this.deactivateControl = function(){
      self.debug = false;
      self.SocketConnector.disconnectSocket();
    };
    this.sendControlLog = function(ori, name, text){
      if(typeof(self.SocketConnector) !== "undefined"){
        self.SocketConnector.logsSocket(ori, (new Date()).toTimeString().split(' ')[0] +" .. " + name+ "--"+text);
      }
    };
    this.changeCore = function(id){
      self.SocketConnector.changeCore(id);
    };
    this.saveIdContent = function(idContent){
        self.id_content_now = idContent;
    };
    this.get_tunnels_params = function(){
        self.api_connector.get(self.api_url,self.api_ver,"player/get_tunnels_params/id",self.core_id,self.remoteResponse);
    };
    this.getInfoSystem = function(){
    }

    this.setContentRotation = function(options, callback_setContentRotation) {
    }

    this.getDisplayOrientation = function(callback,params) {

    }

    this.fileHash = function(filename,callback,params){
      // TODO
      if(typeof(callback) !== "undefined"){
          callback("0",filename,params);
      }
    };

    this.getLIC = function(callback){
      self.readFile("admira.lic", function(resultR, file){
        if(resultR){
          callback("admira.lic = "+resultR);
        }
        else{
          callback("error read admira.lic");
        }
      });
    }
    this.createLIC = function(data, callback){
      /*self.writeFile("admira.lic", data.lic, function(resultR, file){
        if(resultR){
          callback("admira.lic ("+data.lic+") created");
        }
        else{
          callback("admira.lic not created");
        }
      });*/
      callback("no function");
    }
    this.deleteAll = function(callback){
      callback("no function");
    };
    this.deleteAllAll = function(callback){
      callback("no function");
    };
    this.getInfoSystemPlus = function(callback){
      callback("no function");
    };
    this.getTimers = function(callback){
      callback("no function");
    };
    this.clearCache = function(callback) {
      callback("no function");
    };
    this.calculateReboot = function(callback) {
      callback("no function");
    };
    this.copyUSB = function (callback){
      callback("no function");
    };
    this.deleteAll2 =  function(callback){
      callback("no function");
    };
    this.upgradeApplication = function (callback_upgradeApplication){
      callback_upgradeApplication("no function");
    };
    this.upgradeApplicationInsert = function(url, callback_upgradeApplicationInsert){
        callback_upgradeApplicationInsert("no function");
    }
    this.upgradeApplicationInstall = function(callback_upgradeApplicationInstall){
        callback_upgradeApplicationInstall("no function");
    }
    this.upgradeApplicationNow = function(callback_upgradeApplicationNow){
        callback_upgradeApplicationNow("no function");
    }
    this.updateApplicationS = function(callback_updateApplicationS){
        callback_updateApplicationS("no function");
    }
    this.generateNewScreenController = function ( screen ){
    	var controller = false;
		return controller;
	}
    this.updateFirmware = function(url, callback_updateFirmware){
      callback_updateFirmware("no function");
    };
    this.getStatusPanel = function(url, callback_getStatusPanel){
      callback_getStatusPanel("no function");
    };
    this.setPanelOff = function(url, callback_setPanelOff){
      if(typeof(callback_setPanelOff) != "undefined"){
        callback_setPanelOff("Panel Off by suspend system android");
      }
    };
    this.setPanelOn = function(url, callback_setPanelOn){
      if(typeof(callback_setPanelOn) != "undefined"){
        callback_setPanelOn("no function");
      }
    };
    this.changePictureMode = function(callback_changePictureMode){
        callback_changePictureMode("no function");
    }
    this.getPictureProperty = function(callback_getPictureProperty){
        callback_getPictureProperty("no function");
    }
    this.setPortraitMode = function(data, callback_setPortraitMode){
        callback_setPortraitMode("no function");
    }
    this.getContentRotation = function(callback_getContentRotation){
        callback_getContentRotation("no function");
    }
    this.setContentRotation = function(data, callback_setContentRotation){
        callback_setContentRotation("no function");
    }
    this.getVideoStatus = function(callback_getVideoStatus){
        callback_getVideoStatus("no function");
    }
    this.checkSize = function(filename,callback,params){
        callback(false)
    };
    this.shellExec = function(command,callback,params){
      var result = "";
      if(typeof(callback) !== "undefined"){
          callback(result,command,params);
      }
    };

    this.findSyncIp = function (callback){
        window.RTCPeerConnection = window.RTCPeerConnection || window.mozRTCPeerConnection || window.webkitRTCPeerConnection;
        var pc = new RTCPeerConnection({iceServers:[]}), noop = function(){};
        pc.createDataChannel("");
        pc.createOffer(pc.setLocalDescription.bind(pc), noop);
        pc.onicecandidate = function(ice){  //listen for candidate events
            if(!ice || !ice.candidate || !ice.candidate.candidate)  return;
            var ip = /([0-9]{1,3}(\.[0-9]{1,3}){3}|[a-f0-9]{1,4}(:[a-f0-9]{1,4}){7})/.exec(ice.candidate.candidate)[1];
            pc.onicecandidate = noop;
            if(typeof(callback) !== "undefined"){
              callback(ip);
            }
        };
    };
    this.getPowerOnOffHistory = function(callback_getPowerOnOffHistory){
        callback_getPowerOnOffHistory("no function");
    }
    this.getBrigthness = function(callback_getBrigthness){
        callback_getBrigthness("no function");
    }
    this.setBrightness = function(brightness, callback_setBrightness){
        callback_setBrightness("no function");
    }
    this.getContrast = function(callback_getContrast){
        callback_getContrast("no function");
    }
    this.setContrast = function(contrast, callback_setContrast){
        callback_setContrast("no function");
    }
    this.getBacklight = function(callback_getBacklight){
        callback_getBacklight("no function");
    }
    this.setBacklight = function(backlight, callback_setBacklight){
        callback_setBacklight("no function");
    }
    this.geConfigPanel = function(callback_geConfigPanel){
        callback_geConfigPanel("no function");
    }
    this.setConfigPanel = function(data, callback_setConfigPanel){
        callback_setConfigPanel("no function");
    }
    this.setNativePortraitMode = function(data, mode, callback_setNativePortraitMode){
        callback_setNativePortraitMode("no function");
    }
    this.getStatusRemoteControl = function(callback_getStatusRemoteControl){
      callback_getStatusRemoteControl("no function")
    }
    this.setStatusRemoteControl = function(data, callback_setStatusRemoteControl){
        callback_setStatusRemoteControl("no function");
    }
}
ChromeSystemController.prototype = new SystemController();
