function CordovaSystemController(){
    if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
	}
	arguments.callee._singletonInstance = this;

	var self = this; self.name = "CordovaSystemController"; // MANDATORY
    this.rootDir = "";
    this.appDir = "";
    this.sdcardDir = "";
    this.ready_callback = false;
    this.mainDirName = "ADmira/";
    this.fs = false;

    this.uses_local_screenshot = "";
    this.local_screenshot_is_dir = false;
    this.local_screenshots_locations = ["/philips/Screenshots/"]; // Added Philips Android Screenshots Support

    this.systemIsUpdating = false;

    this.websocket_server = false;//cordova.plugins.wsserver;
    this.websocket_port = 0;
    this.websocket_connections = false; // Has to be an array of connections []
    this.websocket_active = false;
    this.initStartSocket = false;
    this.websocket_onOpenCallback = false;
    this.websocket_onMessageCallback = false;
    this.websocket_onTerminatedCallback = false;


    this.dataInfoSystem = null;
    this.sendInfoSystem = false;
    this.superEco = false;
    this.eco = false;
    this.screenshotServer = null;
    this.id_content_now = null;

    this.busy = false;

    this.SocketConnector = undefined;

    this.sync_master_supported = true;

    this.init = function(callback){
        self.SocketConnector = new SocketConnector(self);
        self.ready_callback = callback;
        if(!self.fs){
            var userAgent = navigator.userAgent || navigator.vendor || window.opera;
            userAgent = userAgent.toLowerCase();
            if(userAgent.match(/iphone/i)
              || userAgent.match(/ipad/i)
              || userAgent.match(/ipod/i)){
                if(self.platform == "WEB"){self.platform = "IOS";}
                self.rootDir = cordova.file.documentsDirectory;
            }else if(userAgent.match(/android/i)){
              if(self.platform == "WEB"){self.platform = "ANDROID";}
              self.rootDir = cordova.file.externalRootDirectory;
            }
            self.sdcardDir = self.rootDir;
            window.requestFileSystem  = window.requestFileSystem || window.webkitRequestFileSystem;
            window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, self.onInitFs, self.onErrorFsInit);
            StatusBar.hide();
        }else{
            self.ready_callback();
        }
    };

    this.onQuotaGranted = function(grantedBytes){
        console.log("Granted Bytes : "+grantedBytes);
        console.log("checking folders .... asked");
    };

    this.onQuotaError = function(e){
        console.log("Quota error ",e);
    };

    this.createTrayIcon = function(){
        // none
    };

    this.onReady = function(){
        self.ready_callback();
        self.installExternal();
    };

    this.checkFolders = function(){
        var allExist = true;
        for(var route in self.ROUTES){
            if(self.ROUTES[route].isFolder && !self.ROUTES[route].exists){
                allExist = false;
                self.makeDir(self.ROUTES[route].path,function(){
                   self.ROUTES[route].exists = true;
                });
                break;
            }
        }
        if(allExist){
            self.checkCoreFile();
        }else{
            setTimeout(self.checkFolders,150);
        }
    };

    this.checkCoreFile = function(){
        self.fileExists(self.ROUTES['CORE'].path,function(exists,file_ref){
            if(exists){
                self.checkConfigFile();
            }else{
                self.createCoreFile();
            }
        });
    };

    this.createCoreFile = function(){
        var xml_data = '<?xml version="1.0" encoding="utf-8" ?><core><config><id>0</id><api_ver>3.0</api_ver><api_url>apilb.admira.com</api_url></config><server url="apilb.admira.com"/></core>';
        self.writeFile(self.ROUTES['CORE'].path,xml_data,function(){
            self.checkConfigFile();
        });
    };

    this.checkConfigFile = function(){
        self.fileExists(self.ROUTES['CONFIG'].path,function(exists,file_ref){
            if(exists){
                self.checkDataXmlFile();
            }else{
                self.createConfigFile();
            }
        });
    };

    this.createConfigFile = function(){
        var xml_data = '<config><data file="datos.xml"/><paths multiplayer_delimiter="_" xmls="xmls/" content="content/" temp="temp/" modules="modules/" log="log/" stadistics="stats/"/><params bumper_content="assets/downloading.swf" bumper_duration="56" alwaysontop="1"/></config>';
        self.writeFile(self.ROUTES['CONFIG'].path,xml_data,function(){
            self.checkDataXmlFile();
        });
    };

    this.checkDataXmlFile = function(){
        var filename = self.getDayOfWeek()+".xml";
        self.fileExists(self.ROUTES['XML'].path+filename,function(exists,file_ref){
            if(exists){
                self.onReady();
            }else{
                self.createDataXmlFile();
            }
        });
    };

    this.createDataXmlFile = function(){
        var filename = self.getDayOfWeek()+".xml";
        var xml_data = '<?xml version="1.0"?><admira timestamp="1" version="1.0" status="ok"><archive/><playlist><loop/></playlist><screens /><config volume="100" fullscreen="true" mode="1" always_on_top="true" plv_name="Debug Name" project_name="Debug Project" time="30" plv_id="0" quit_on_click="true" activated="false" domain="" api_key="" live_url="" live_recheck_seconds="" live_send_id=""><resolution x="100" y="100" width="300" height="300" /></config></admira>';
        self.writeFile(self.ROUTES['XML'].path+filename,xml_data,function(){
            self.onReady();
        });
    }

    this.onInitFs = function(fs){
        self.fs = fs;
        self.makeDir(self.mainDirName,function(file_ref,created){
            self.rootDir = self.rootDir+self.mainDirName;
            setTimeout(self.checkFolders,150);
        });

        // check local screenshot based on /sdcard
        for(var i=0;i<self.local_screenshots_locations.length;i++){
            self.log("searching : "+self.sdcardDir);
            window.resolveLocalFileSystemURL(self.sdcardDir+self.local_screenshots_locations[i],function(fileEntry){
                self.uses_local_screenshot = fileEntry.nativeURL;
                self.local_screenshot_is_dir = fileEntry.isDirectory;
            });
        }
    };

    this.onErrorFsInit = function(e){
        self.log("Filesystem error ",e);
    };

    this.onResolveError = function(e){
        self.log("Resolve Path error ",e);
    };

    this.onFileError = function(e){
        // https://developer.mozilla.org/en-US/docs/Web/API/FileError
        var msg = '';
        switch (e.code) {
            case FileError.QUOTA_EXCEEDED_ERR:
                msg = 'Storage quota exceeded';
                break;
            case FileError.NOT_FOUND_ERR:
                msg = 'File not found';
                break;
            case FileError.SECURITY_ERR:
                msg = 'Security error';
                break;
            case FileError.INVALID_MODIFICATION_ERR:
                msg = 'Invalid modification';
                break;
            case FileError.INVALID_STATE_ERR:
                msg = 'Invalid state';
                break;
            default:
                msg = 'Unknown error';
                break;
        };
        //console.log("File ERROR : "+msg);
        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+1,self.name+": IO File error: "+msg);
    };

    this.installExternal = function(){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            // CHECK IF EXTERNAL APP EXISTS READING admira.ver FILE (only written when copy is complete)
            directoryEntry.getFile("app/cordova.ver",{create:false}, function (fileEntry) {
                self.log(self.rootDir+"app exists");
            }, function(e){
              directoryEntry.getDirectory("app/",{create:false}, function (dirEntry) {
                  dirEntry.removeRecursively(function(entry){
                      self.log("app/ removed ... not valid");
                      self.copyExternalApp(directoryEntry);
                  },function(e){
                      self.log("imposible to remove app/");
                  });
              }, function(e){
                  self.copyExternalApp(directoryEntry);
                  self.log("app/ not found");
              });

            });
        }, self.onResolveError.bind(null, ""));
    };

    this.copyExternalApp = function(directoryEntry){
      window.resolveLocalFileSystemURL(cordova.file.applicationDirectory,function(a){
          a.getDirectory("www/",{create: false}, function (b) {
              b.copyTo(directoryEntry,"app", function(entry){
                  self.readFile("app/cordova.ver",function(data,file_ref,params){
                    var version = self.platform;
                    if(data != ""){
                      version = data;
                    }
                    self.writeFile(self.ROUTES['VER'].path,version,function(ok,file_ref,params){
                      self.log("installExternal completed. app is now ready for remote updates ...");
                    });
                  });
              }, function(error){
                  self.log("installExternal error " + error);
              });
          }, function(e){
              self.log(cordova.file.applicationDirectory+"www does NOT exists");
          });
      }, self.onResolveError.bind(null, ""));
    };

    this.writeFile = function(filename,value,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create: true},function(fileEntry){
                fileEntry.createWriter(function (fileWriter) {
                    fileWriter.onwriteend = function (e) {
                        if (fileWriter.length === 0) {
                            fileWriter.write(value);
                            if(typeof(callback) !== "undefined"){
                                callback(true,filename,params);
                            }
                        }
                    };
                    fileWriter.onerror = function(e){
                      self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+2,self.name+"Write File "+filename+" Error : "+e.toString());
                      if(typeof(callback) !== "undefined"){
                          callback(false,filename,params);
                      }
                      //this.onerror = null;
                    };
                    fileWriter.truncate(0);
                }, self.onFileError.bind(null, filename));
            }, self.onFileError.bind(null, filename));
        }, self.onResolveError.bind(null, filename));
    };

    this.writeBlob = function(filename,blob,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create: true},function(fileEntry){
                fileEntry.createWriter(function (fileWriter) {
                    fileWriter.onwriteend = function (e) {
                        if (fileWriter.length === 0) {
                            fileWriter.write(blob);
                            if(typeof(callback) !== "undefined"){
                                callback(true,filename,params);
                            }
                        }
                    };
                    fileWriter.onerror = function(e){
                        if(typeof(callback) !== "undefined"){
                            callback(false,filename,params);
                        }
                    };
                    fileWriter.truncate(0);
                }, self.onFileError.bind(null, filename));
            }, self.onFileError.bind(null, filename));
        }, self.onResolveError.bind(null, filename));
    };

    this.appendFile = function(filename,value,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create: true},function(fileEntry){
                fileEntry.createWriter(function (fileWriter) {
                    fileWriter.seek(fileEntry.length);
                    fileWriter.onwriteend = function (e) {
                        if(typeof(callback) !== "undefined"){
                            callback(true,filename,params);
                        }
                    };
                    fileWriter.onerror = function(e){
                        self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+3,self.name+"Append File "+filename+" Error : "+e.toString());
                        if(typeof(callback) !== "undefined"){
                            callback(false,filename,params);
                        }
                    };
                    fileWriter.write(value);
                }, self.onFileError.bind(null, filename));
            }, self.onFileError.bind(null, filename));
        }, self.onResolveError.bind(null, filename));
    };

    this.readFile = function(filename, callback, params){
        var pathToFile = self.rootDir + filename;
        window.resolveLocalFileSystemURL(pathToFile,function(fileEntry){
            fileEntry.file(function(file){
                var reader = new FileReader();
                reader.onloadend = function (e) {
                    var value = e.target.result;
                    if(typeof(callback) !== "undefined"){
                        callback(value,filename,params);
                    }
                };
                reader.onerror = function (e) {
                    if(typeof(callback) !== "undefined"){
                        callback("",filename,params);
                    }
                };
                reader.readAsText(file);
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback("",filename,params);
                }
            });
        },function(e){
            if(typeof(callback) !== "undefined"){
                callback("",filename,params);
            }
        });
    };

    this.fileHash = function(filename,callback,params){
      if(self.platform == "ANDROID"){
        try{
          var stored_hash = "";
          if(typeof(params) != "undefined"){
            if(typeof(params.hash) != "undefined"){
              stored_hash = params.hash;
            }
          }
          var options = {
            "filename":self.rootDir + filename,
            "hash": stored_hash
          }
          cordova.plugins.backgroundMode.getFileHash(options,function(result){
            self.log(result);
            if(typeof(callback) !== "undefined"){
                if(typeof(result.hash) != "undefined"){
                  callback(result.hash,filename,params);
                }else{
                  callback("",filename,params);
                }
            }
          });
        }catch(e){
          if(typeof(callback) !== "undefined"){
              callback("NONE",filename,params);
          }
        }
      }else{
        if(typeof(callback) !== "undefined"){
            callback("NONE",filename,params);
        }
      }
    };

    this.fileExists = function(filename,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create:false}, function (fileEntry) {
                if(typeof(callback) !== "undefined"){
                    callback(true,filename,params);
                }
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
            });
        }, self.onResolveError.bind(null, filename));
    };

    this.dirExists = function(dirname,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getDirectory(dirname,{create:false}, function (entry) {
                if(typeof(callback) !== "undefined"){
                    callback(true,dirname,params);
                }
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,dirname,params);
                }
            });
        }, self.onResolveError.bind(null, dirname));
    };

    this.makeDir = function(path,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getDirectory(path,{create:true, exclusive:false}, function (directoryEntry) {
                if(typeof(callback) !== "undefined"){
                    callback(true,path,params);
                }
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,path,params);
                }
            });
        }, self.onResolveError.bind(null, path));
    };

    this.makeDirRecursive =function(path, callback, params, position)
	{
		position = (typeof position == 'undefined')? 0: position;

		var path_split 		= path.split('/');
		var new_position 	= position+1;
		var sub_path 		= path_split.slice(0,new_position).join('/');

		var inner_callback = function(obj){
			return function(){
				obj.makeDirRecursive(path, callback, params, new_position);
			}
		}

		if(new_position == path_split.length){
			self.makeDir(sub_path, callback, params);
		}else{
			self.makeDir(sub_path, inner_callback(self), params);
		}
	};

    this.modificationDate = function(filename,callback,params){
        var pathToFile = self.rootDir + filename;
        window.resolveLocalFileSystemURL(pathToFile, function (fileEntry) {
            fileEntry.getMetadata(function(metadata){
                var date = new Date(metadata.modificationTime).getTime();
                if(typeof(callback) !== "undefined"){
                    callback(date,filename,params);
                }
            }, function(error){
                if(typeof(callback) !== "undefined"){
                    callback(-1,filename,params);
                }
            });
        }, self.onResolveError.bind(null, filename));
    };

    this.downloadFile = function(url,destFolder,callback,params){
        var filename = url;
        var stored_hash = "";
        filename = filename.substring(filename.lastIndexOf("/")+1,filename.length);
        if(typeof(params) !== "undefined"){
            if(typeof(params.filename) !== "undefined"){
                filename = params.filename;
            }
            if(typeof(params.hash) !== "undefined"){
                stored_hash = params.hash;
            }
        }
        var fileTransfer = new FileTransfer();
        var uri = encodeURI(url);

        fileTransfer.download(uri,self.rootDir+self.ROUTES['TMP'].path+filename,function(fileEntry) {
          if(stored_hash.length!=40){
            self.log("SHA1 Check unneeded "+filename+" has no stored hash to check ...");
            window.resolveLocalFileSystemURL(self.rootDir+destFolder,function(directoryEntry) {
                fileEntry.moveTo(directoryEntry, filename, function(entry){
                    if(typeof(callback) !== "undefined"){
                        callback(true,url,params);
                    }
                }, function(error){
                     self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+"download "+url+" error " + error);
                    if(typeof(callback) !== "undefined"){
                        callback(false,url,params);
                    }
                });
            },self.onResolveError.bind(null, filename));
          }else{
            self.fileHash(self.ROUTES['TMP'].path+filename,function(hash,file_ref,params){
              self.log("SHA1 Check XML = "+stored_hash+" Downloaded : "+hash);
              if(hash != stored_hash && hash != "NONE"){
                self.log("SHA1 Check failed ...  XML = "+stored_hash+" Downloaded : "+hash+" for file : "+filename);
                self.deleteFile(self.rootDir+self.ROUTES['TMP'].path+filename);
                self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+": Function downloadFile() SHA1 Check failed ... XML = "+stored_hash+" Downloaded : "+hash+" for file : "+filename)
                if(typeof(callback) !== "undefined"){
                    callback(false,url,params);
                }
              }else{
                window.resolveLocalFileSystemURL(self.rootDir+destFolder,function(directoryEntry) {
                    fileEntry.moveTo(directoryEntry, filename, function(entry){
                        self.log("Download complete : "+filename);
                        if(typeof(callback) !== "undefined"){
                            callback(true,url,params);
                        }
                    }, function(error){
                         self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+4,self.name+"download "+url+" error " + error);
                        if(typeof(callback) !== "undefined"){
                            callback(false,url,params);
                        }
                    });
                },self.onResolveError.bind(null, filename));
              }
            },params);
          }
        },function(error){
            if(typeof(callback) !== "undefined"){
                callback(false,url,params);
            }
        },false);
    };

    this.unzipFile = function(filename,destFolder,callback,params){
      if(typeof(zip) != "undefined"){
        // JAVA NATIVE COMPATIBLE UNZIP SYSTEM
        self.log("UNZIP : From "+self.rootDir+filename+" to "+self.rootDir+destFolder);
        zip.unzip(self.rootDir+filename,self.rootDir+destFolder,function(correct){
          if(correct == 0){
            self.log("UNZIP : Completed");
            if(typeof(callback) !== "undefined"){
                callback(true,filename,params);
            }
          }else{
            self.log("UNZIP : Failed");
            if(typeof(callback) !== "undefined"){
                callback(false,filename,params);
            }
          }
        },function(progressEvent){
          self.log("UNZIP : Extracting ... "+Math.round((progressEvent.loaded / progressEvent.total) * 100)+"%");
        });
      }else{
        // OLD PURE JAVASCRIPT UNZIP SYSTEM
        self.makeDirRecursive(destFolder,function(created,file_ref){
            JSZipUtils.getBinaryContent(self.rootDir+filename, function(err, data) {

                if(err) {
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+5,self.name+"unzip error File : "+filename);
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                    throw err; // or handle err
                }

                var zip = new JSZip(data);
                for(var file in zip.files){
                    if(zip.files[file].dir){
                      self.log("Trying to unzip file : "+file);
                      if(file == "node_modules"){
                          self.log("avoid unneeded folder");
                      }else{
                        self.makeDirRecursive(destFolder+file);
                      }
                    }
                }
                self.checkZipDirectoryStructure(zip,destFolder,callback,params);
            });
        });
      }
    };

    this.checkZipDirectoryStructure = function(zip,destFolder,callback,params){
        var all_exists = true;
        for(var file in zip.files){
             if(zip.files[file].dir){
                 if(!zip.files[file].options.binary){
                     self.dirExists(destFolder+file,function(exists,file_ref){
                         zip.files[file].options.binary = exists;
                     });
                     all_exists = false;
                     break;
                 }
             }
        }

        if(all_exists){
            for(var file in zip.files){
                var filename = file.substring(file.lastIndexOf("/")+1,file.length);
                var first_char_filename = filename.substring(0,1);
                var ext = file.substring(file.lastIndexOf(".")+1,file.length);
                if(ext == "exe" || ext == "zip" || ext == "EXE" || ext == "dll" || ext == "wgt" || ext == "sh"){
                    self.log("file is not valid : "+file);
                }else if(first_char_filename == "."){
                    self.log("file is hidden file: "+file);
                }else if(zip.files[file].dir){
                    self.log("no need , is dir");
                }else{
                    self.writeBlob(destFolder+file,zip.files[file].asBinary());
                }
            }
            if(typeof(callback) !== "undefined"){
                callback(true,file,params);
            }
        }else{
            self.log("Recheck : createZipDirectoryStructure");
            setTimeout(self.checkZipDirectoryStructure.bind(null,zip,destFolder,callback,params),150);
        }
    };



    this.readDirectory = function(directory,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir+directory,function(directoryEntry){
            var reader = directoryEntry.createReader();
            var files = [];
            reader.readEntries (function(fileEntries) {
                for(var i=0;i<fileEntries.length;i++){
                    files.push(fileEntries[i].name);
                }
                if(typeof(callback) !== "undefined"){
                    callback(files,directory,params);
                }
            },function(e){
                if(typeof(callback) !== "undefined"){
                    callback(files,directory,params);
                }
            });
        }, self.onResolveError.bind(null, directory));
    };

    this.watchDirectory = function(directory,compare_files,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir+directory,function(directoryEntry){
            var reader = directoryEntry.createReader();
            reader.readEntries (function(fileEntries) {
                var checkMetadata = function(index,entries) {
                    return function(fileMetadata) {
                        files[index].mdate = new Date(fileMetadata.modificationTime).getTime();
                        index++;
                        if(index < entries.length){
                            fileEntries[index].getMetadata(checkMetadata(index,entries),function(e){
                                if(typeof(callback) !== "undefined"){
                                    callback(compare_files,[]);
                                }
                            });
                        }else{
                            var compare_index = -1;
                            var changed = new Array();
                            for(var j=0;j<compare_files.length;j++){
                                compare_index = files.map(function(e){return e.filename;}).indexOf(compare_files[j].filename);
                                if(compare_index >= 0){
                                    if(compare_files[j].mdate != files[compare_index].mdate){
                                        //console.log(compare_files[j].mdate+" <> "+files[compare_index].mdate);
                                        compare_files[j].mdate = files[compare_index].mdate;
                                        changed.push(compare_files[j]);
                                    }
                                }else{
                                    if(compare_files[j].mdate != 0){
                                        compare_files[j].mdate = 0;
                                        changed.push(compare_files[j]);
                                    }
                                }
                            }
                            if(typeof(callback) !== "undefined"){
                                callback(compare_files,changed,params);
                            }
                        }
                    };
                };

                var files = new Array(fileEntries.length);
                for(var f=0;f<fileEntries.length;f++){
                    files[f] = null;
                    files[f] = new Object({'filename':fileEntries[f].name,'mdate':0});
                }
                if(typeof(fileEntries[0]) !== "undefined"){
                    fileEntries[0].getMetadata(checkMetadata(0,fileEntries),function(e){
                        if(typeof(callback) !== "undefined"){
                            callback(compare_files,[]);
                        }
                    });
                }
            },function(e){
                if(typeof(callback) !== "undefined"){
                    callback(compare_files,[]);
                }
            });
        }, function(e){
            if(typeof(callback) !== "undefined"){
                callback(compare_files,[]);
            }
        });
    };

    this.deleteFile = function(filename,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getFile(filename,{create:false,exclusive:false}, function (fileEntry) {
                fileEntry.remove(function(entry){
                    if(typeof(callback) !== "undefined"){
                        callback(true,filename,params);
                    }
                },function(e){
                    self.log("Error deleting file ... "+filename);
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+6,self.name+"Error deleting file : "+filename);
                    if(typeof(callback) !== "undefined"){
                        callback(false,filename,params);
                    }
                });
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,filename,params);
                }
            });
        }, self.onResolveError.bind(null, filename));
    };


    this.deleteDirectory = function(dirname,callback,params){
        window.resolveLocalFileSystemURL(self.rootDir,function(directoryEntry){
            directoryEntry.getDirectory(dirname,{create:false}, function (dirEntry) {
                dirEntry.removeRecursively(function(entry){
                    if(typeof(callback) !== "undefined"){
                        callback(true,dirname,params);
                    }
                },function(e){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+6,self.name+"Error deleting directory recursively : "+dirname);
                    if(typeof(callback) !== "undefined"){
                        callback(false,dirname,params);
                    }
                });
            }, function(e){
                if(typeof(callback) !== "undefined"){
                    callback(false,dirname,params);
                }
            });
        }, self.onResolveError.bind(null, dirname));
    };

    this.deleteFilesInDir = function(dir,filter,callback,params){
      if(typeof(filter) === "undefined"){filter = "";}
      window.resolveLocalFileSystemURL(self.rootDir+dir,function(directoryEntry){
            var reader = directoryEntry.createReader();
            var files = [];
            reader.readEntries (function(fileEntries) {
                var deleted_files = [];
                for(var i=0;i<fileEntries.length;i++){
                    if(filter != ""){
                        var index = fileEntries[i].name.lastIndexOf(filter);
                        if(index >= 0){
                            fileEntries[i].remove();
                            deleted_files.push(fileEntries[i].name);
                        }
                    }else{
                        fileEntries[i].remove();
                        deleted_files.push(fileEntries[i].name);
                    }
                }
                if(typeof(callback) !== "undefined"){
                    callback(deleted_files,dir,params);
                }
            },function(e){
                if(typeof(callback) !== "undefined"){
                    callback([],dir,params);
                }
            });
        }, self.onResolveError.bind(null, dir));
    };

    this.moveFile = function(source,destination,callback,params){
        if(typeof(callback) !== "undefined"){
            callback(false,"",params);
        }
    };

    this.systemUpdate = function(update_folder,platform){
        if(!self.systemIsUpdating){
            self.systemIsUpdating = true;
            self.log("updating system ... ");
            window.resolveLocalFileSystemURL(self.rootDir,function(root_dir){
                root_dir.getDirectory("app/",{create: false}, function (root_dir_app) {
                    root_dir_app.moveTo(root_dir,"app_backup", function(root_dir_app_backup){
                        self.log("moved app to app_backup complete");
                        window.resolveLocalFileSystemURL(self.rootDir+update_folder+"files/",function(files){
                            files.getDirectory("app/",{create: false}, function (files_app) {
                                self.log(cordova.file.applicationDirectory+"www exists");

                                files_app.moveTo(root_dir,"app/", function(entry){
                                    self.log("moved files/app to app complete !!!! ");
                                    root_dir_app_backup.getFile("cordova.js",{create: false}, function (cordova_js) {
                                        cordova_js.copyTo(root_dir_app,"cordova.js", function(entry){
                                            self.log("moved cordova.js to app complete !!!! ");
                                        });
                                    });
                                    root_dir_app_backup.getFile("cordova_plugins.js",{create: false}, function(cordova_plugins_js) {
                                        cordova_plugins_js.copyTo(root_dir_app,"cordova_plugins.js", function(entry){
                                            self.log("moved cordova_plugins.js to app complete !!!! ");
                                        });
                                    });
                                    root_dir_app_backup.getDirectory("plugins/",{create: false}, function (plugins) {
                                        plugins.copyTo(root_dir_app,"plugins/", function(entry){
                                            self.log("moved plugins to app complete !!!! ");
                                            self.deleteDirectory("temp/upgrade/");
                                            root_dir_app_backup.removeRecursively();
                                            self.systemIsUpdating = false;
                                            setTimeout(function(){
                                              location.reload(true);
                                            },15000);
                                            self.log("Upgrade completed ... reloading app in 15s");
                                            if(typeof(callback) !== "undefined"){
                                                callback(false,"",params);
                                            }
                                        });
                                    });
                                }, function(error){
                                    self.log("moved files/app to app error : " + error);
                                });
                            }, function(e){
                                self.log(self.rootDir+update_folder+"files/ NOT exists");
                            });
                            files.getFile("admira.ver",{},function(admira_version){
                                admira_version.moveTo(root_dir,"admira.ver", function(entry){
                                    self.log("admira ver installed");
                                });
                            });
                        }, self.onResolveError.bind(null, ""));
                    }, function(error){
                        self.log("moved app to app_backup error : " + error);
                    });
                }, function(e){
                    self.log("main app dir does NOT exists");
                });
            }, self.onResolveError.bind(null, ""));
        }
    };

    this.systemRepairUpdate = function(){
      // TODO
    };

    this.takeScreenshot = function(callback,params){
        if(self.uses_local_screenshot == ""){
            navigator.screenshot.URI(function(error,res){
                if(error){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+7,self.name+": screenshot error: "+error.toString());

                    if(typeof(callback) !== "undefined"){
                        callback(false,params);
                    }
                }else{
                    if(typeof(res !== "undefined")){
                        var img = new Image();
                        img.crossOrigin = 'Anonymous';
                        img.onload = function(){
                            var base64 = false;
                            var canvas = document.createElement('CANVAS');
                            var context = canvas.getContext('2d');
                            //250x140 Screenshot size
                            var width = 250;
                            var height = 140;
                            if (img.height > img.width){
                                width = 140;
                                height = 250;
                            }
                            canvas.height = height;
                            canvas.width = width;
                            context.drawImage(img, 0, 0,width,height);
                            if(params){
                                base64 = canvas.toDataURL("image/jpeg", params);
                            }
                            else{
                                base64 = canvas.toDataURL("image/jpeg", 1.0);
                            }
                            if(typeof(callback) !== "undefined"){
                                self.screenshotServer = base64.substring(23);
                                callback(base64.substring(23),params);
                            }
                            canvas = null;
                        };
                        img.src = res.URI;
                    }else{
                        if(typeof(callback) !== "undefined"){
                            callback(false,params);
                        }
                    }
                }
            },75);
        }else{
            if(!self.local_screenshot_is_dir){
                var img = new Image();
                img.crossOrigin = 'Anonymous';
                img.onload = function(){
                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    context.drawImage(img, 0, 0,width,height);
                    if(params){
                        base64 = canvas.toDataURL("image/jpeg", params);
                    }
                    else{
                        base64 = canvas.toDataURL("image/jpeg", 1.0);
                    }
                    if(typeof(callback) !== "undefined"){
                        callback(base64.substring(23),params);
                    }
                    canvas = null;
                };
                var d = new Date();
                img.src = self.uses_local_screenshot+"?time="+d;
            }else{
                window.resolveLocalFileSystemURL(self.uses_local_screenshot,function(directoryEntry){
                    var reader = directoryEntry.createReader();
                    reader.readEntries (function(fileEntries) {
                        var selected = "";
                        if(fileEntries.length > 0){
                            selected = fileEntries[fileEntries.length-1].nativeURL;
                        }


                        var img = new Image();
                        img.crossOrigin = 'Anonymous';
                        img.onload = function(){
                            var base64 = false;
                            var canvas = document.createElement('CANVAS');
                            var context = canvas.getContext('2d');
                            //250x140 Screenshot size
                            var width = 250;
                            var height = 140;
                            if (img.height > img.width){
                                width = 140;
                                height = 250;
                            }
                            canvas.height = height;
                            canvas.width = width;
                            context.drawImage(img, 0, 0,width,height);
                            if(params){
                                base64 = canvas.toDataURL("image/jpeg", params);
                            }
                            else{
                                base64 = canvas.toDataURL("image/jpeg", 1.0);
                            }
                            for(var i=0;i<fileEntries.length;i++){
                                fileEntries[i].remove(function(){
                                    directoryEntry.remove(function(){
                                        self.log("Remove Screenshots Directory OK");
                                    },function(){
                                        self.log("Remove Screenshots Directory FAILED !");
                                    });
                                });
                            }

                            if(typeof(callback) !== "undefined"){
                                callback(base64.substring(23),params);
                            }
                            canvas = null;

                        };
                        if(selected != ""){
                            var d = new Date();
                            self.log("Trying to capture : "+selected);
                            img.src = selected+"?time="+d;
                        }else{
                             if(typeof(callback) !== "undefined"){
                                callback("",params);
                            }
                        }
                    },function(e){
                        console.log(e);
                    });
                });
            }
        }
    };
    this.getScreenshot = function(callback){
        if(self.uses_local_screenshot == ""){
            navigator.screenshot.URI(function(error,res){
                if(error){
                    self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+7,self.name+": screenshot error: "+error.toString());

                    if(typeof(callback) !== "undefined"){
                        callback(false);
                    }
                }else{

                    if(typeof(res !== "undefined")){
                        if(typeof(callback) !== "undefined"){
                            callback(res.URI.substring(23));
                        }
                    }else{
                        if(typeof(callback) !== "undefined"){
                            callback(false);
                        }
                    }
                }
            },75);
        }else{
            if(!self.local_screenshot_is_dir){
                var img = new Image();
                img.crossOrigin = 'Anonymous';
                img.onload = function(){
                    var base64 = false;
                    var canvas = document.createElement('CANVAS');
                    var context = canvas.getContext('2d');
                    //250x140 Screenshot size
                    var width = 250;
                    var height = 140;
                    if (img.height > img.width){
                        width = 140;
                        height = 250;
                    }
                    canvas.height = height;
                    canvas.width = width;
                    context.drawImage(img, 0, 0,width,height);
                    base64 = canvas.toDataURL("image/jpeg", 1.0);
                    if(typeof(callback) !== "undefined"){
                        callback(base64.substring(23));
                    }
                    canvas = null;
                };
                var d = new Date();
                img.src = self.uses_local_screenshot+"?time="+d;
            }else{
                window.resolveLocalFileSystemURL(self.uses_local_screenshot,function(directoryEntry){
                    var reader = directoryEntry.createReader();
                    reader.readEntries (function(fileEntries) {
                        var selected = "";
                        if(fileEntries.length > 0){
                            selected = fileEntries[fileEntries.length-1].nativeURL;
                        }


                        var img = new Image();
                        img.crossOrigin = 'Anonymous';
                        img.onload = function(){
                            var base64 = false;
                            var canvas = document.createElement('CANVAS');
                            var context = canvas.getContext('2d');
                            //250x140 Screenshot size
                            var width = 250;
                            var height = 140;
                            if (img.height > img.width){
                                width = 140;
                                height = 250;
                            }
                            canvas.height = height;
                            canvas.width = width;
                            context.drawImage(img, 0, 0,width,height);
                            base64 = canvas.toDataURL("image/jpeg", 1.0);

                            for(var i=0;i<fileEntries.length;i++){
                                fileEntries[i].remove(function(){
                                    directoryEntry.remove(function(){
                                        self.log("Remove Screenshots Directory OK");
                                    },function(){
                                        self.log("Remove Screenshots Directory FAILED !");
                                    });
                                });
                            }

                            if(typeof(callback) !== "undefined"){
                                callback(base64.substring(23));
                            }
                            canvas = null;

                        };
                        if(selected != ""){
                            var d = new Date();
                            img.src = selected+"?time="+d;
                        }else{
                             if(typeof(callback) !== "undefined"){
                                callback(false);
                            }
                        }
                    },function(e){
                        self.log(e);
                    });
                });
            }
        }
    };

    this.setSystemDateTime = function(timestamp,callback,params){
        // TODO
    };

    this.rebootSystem = function(){
        if(!self.systemIsUpdating){
            cordova.plugins.Reboot.reboot();
        }
    };

    this.haltSystem = function(){
        setInterval(function(){
          if(!self.systemIsUpdating){
              try{
                cordova.plugins.Reboot.poweroff();
              }catch(e){
                self.log("Power off not implemented");
              }
          }
        },300);
    };

    this.internetConnected = function() {
        var internet = true;
        var networkState = navigator.connection.type;
        var states = {};
        states[Connection.UNKNOWN] = 'Unknown connection';
        states[Connection.ETHERNET] = 'Ethernet connection';
        states[Connection.WIFI] = 'WiFi connection';
        states[Connection.CELL_2G] = 'Cell 2G connection';
        states[Connection.CELL_3G] = 'Cell 3G connection';
        states[Connection.CELL_4G] = 'Cell 4G connection';
        states[Connection.CELL] = 'Cell generic connection';
        states[Connection.NONE] = 'No network connection';

        if(networkState == Connection.UNKNOWN || networkState == Connection.NONE){
            internet = false;
        }
        self.log("Internet Status : "+states[networkState]+" has internet ? "+internet);
        return internet;
    };

    this.getNetworkInfo = function(callback,params){
        var ip_mode = 1;

        var networkState = navigator.connection.type;

        var network_info = {
            ip: "0.0.0.0",
            netmask : "0.0.0.0",
            gateway : "0.0.0.0"
        };

        if(self.websocket_server){
            self.websocket_server.getInterfaces(function(result){
                for(var interface in result){
                    if (result.hasOwnProperty(interface)) {
                        /*
                        console.log('interface', interface);
                        console.log('ipv4', result[interface].ipv4Addresses);
                        console.log('ipv6', result[interface].ipv6Addresses);
                        console.log("Local Ip : "+network_info.ip);
                        */
                        network_info.ip = result[interface].ipv4Addresses[0];
                        if(typeof(callback) !== "undefined"){
                            callback(network_info,params);
                        }
                    }
                }
            },function(){
                if(typeof(callback) !== "undefined"){
                    callback(network_info,params);
                }
            });
        }
    };

    this.getScreens = function(){
        // OVERRIDE
        var screens = [];
        screens[0] = {id:0,bounds:{x:0,y:0,width:400,height:300,scaleFactor:1}};
        return screens;
    };

    this.setWindow = function(x,y,w,h){
        // OVERRIDE
    };

    this.setProxyConfig = function(config){
        // OVERRIDE
    };

    this.getProxyForURL = function(url){
        // OVERRIDE
        return "";
    };

    this.getAddWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getRemoveWhiteList = function(sourceOrigin, destinationProtocol, destinationHost, allowDestinationSubdomains){
        // OVERRIDE
    };

    this.getDataPath = function(){
        return self.rootDir;
    };

    this.setCrashDumpPath = function(folderPath){
         // OVERRIDE
    };

    this.quit = function(stop,kill){
        if(typeof(stop) === "undefined"){stop = false;}
        if(typeof(kill) === "undefined"){kill = false;}
        kill = stop && kill;
        if(!self.systemIsUpdating){
            if(self.platform == "ANDROID"){
                if(kill){
                  self.shellExec("su -c am force-stop com.admira.player");
                }else if(stop){
                  self.force_screensaver_mode();
                }else{
                  navigator.app.exitApp();
                }
            }
        }
    };
    
    this.force_screensaver_mode = function(minutes){
      if(typeof(minutes) === "undefined"){minutes = 5;}
      minutes = minutes * 60;
      var dow = self.getDayOfWeek();
      var timestamp = parseInt(self.getDate().getTime()/1000);
      var actual_time = 0;
      self.readFile("xmls/"+dow+".xml",function(data){
        data = data.replace('timestamp="'+timestamp+'"','timestamp="1"');
        data = data.replace('mode="0"','mode="1"');
        data = data.replace('mode="2"','mode="1"');
        data = data.replace('mode="3"','mode="1"');
        var actual_time = data.substring(data.lastIndexOf('time="'),data.lastIndexOf('" timezone'));
        actual_time = actual_time.substring(actual_time.lastIndexOf('="')+2,actual_time.length);
        data = data.replace('time="'+actual_time+'"','time="'+minutes+'"');
        self.writeFile("xmls/"+dow+".xml",data,function(ok){
          self.writeFile("xmls/week.stamp","1");
          navigator.app.exitApp();
        });
      });
    };

    this.clearCache = function(callback){
        if(self.platform == "ANDROID"){
          self.shellExec("su -c rm -rf /data/data/com.admira.player/app_xwalkcore/Default/Cache/;am startservice com.admira.player/com.admira.watchdog.ForegroundService;",function(ok){
            if(typeof(callback) !== "undefined"){
              callback(ok);
            }
            self.notifyError(self.ERROR['OPERATING_SYSTEM_CONTROLLER']+8,self.name+": Clear Cache : "+ok);
          });
        }
    };

    this.alwaysOnTop = function(value){
        // OVERRIDE
    };

    this.focusWindow = function(){
        // OVERRIDE
    };

    this.disableRemoteControl = function(platform){
        self.deactivateControl();
    };

    this.enableRemoteControl = function(platform,tunnels){
        self.activateControl();
    };

    this.keepAwake = function(platform){
        // TODO : Keeps Screen Awake
    };

    this.activateControl = function(){
      self.debug = true;
      self.SocketConnector.createSocket();
    };
    this.deactivateControl = function(){
      self.debug = false;
      self.SocketConnector.disconnectSocket();
    };
    this.sendControlLog = function(ori, name, text){
      if(typeof(self.SocketConnector) !== "undefined"){
        self.SocketConnector.logsSocket(ori, (new Date()).toTimeString().split(' ')[0] +" .. " + name+ "--"+text);
      }
    };
    this.changeCore = function(id){
      self.SocketConnector.changeCore(id);
    };
    this.saveIdContent = function(idContent){
        self.id_content_now = idContent;
    };
    this.get_tunnels_params = function(){
        self.api_connector.get(self.api_url,self.api_ver,"player/get_tunnels_params/id",self.core_id,self.remoteResponse);
    };
    this.getInfoSystem = function(){

    }

    this.socketServerClose = function(uuid,error_code,reason){
        if(self.websocket_server){
            self.websocket_server.close({'uuid':uuid},error_code,reason);
        }
    };

    this.socketServerStop = function(){
        if(self.websocket_server){
            self.log("stopping server ... ");
            self.websocket_server.stop(self.onSocketStop,self.onSocketStopFail);
            self.websocket_active = false;
        }
    };

    this.socketServerSend = function(conn,msg){
        if(self.websocket_server){
            self.websocket_server.send(conn,msg);
        }
    };

    this.socketServerBroadcast = function(msg){
        if(self.websocket_server){
              if(self.websocket_connections[0]){
                  for(var client in self.websocket_connections){
                     if(self.websocket_connections[client].state == "open"){
                       self.websocket_server.send(self.websocket_connections[client],JSON.stringify(msg));
                     }
                  }
              }
        }
    };


    this.socketServerStart = function(port,onOpenCallback,onMessageCallback,onTerminatedCallback){
        if(!self.websocket_server){
            self.websocket_server = cordova.plugins.wsserver;
        }
        if(!self.websocket_connections[0]){
            if(!self.initStartSocket){
                self.initStartSocket = true;
                self.log(JSON.stringify(self.websocket_server))
                self.log("Port Param : "+port);
                if(typeof(port) !== "undefined"){self.websocket_port = port;}
                self.log("SocketServerPort : "+self.websocket_port);
                self.websocket_server.start(self.websocket_port,{
                                            'onFailure' : self.onSocketFailure,
                                            'onOpen' : self.onSocketOpen,
                                            'onMessage' : self.onSocketMessage,
                                            'onClose' : self.onSocketClose,
                                            'tcpNoDelay' : false
                                        },self.onSocketStart,self.onSocketStartFail);

                if(typeof(onOpenCallback) !== "undefined"){
                  self.websocket_onOpenCallback = onOpenCallback;
                }
                if(typeof(onMessageCallback) !== "undefined"){
                 self.websocket_onMessageCallback = onMessageCallback;
                }
                if(typeof(onTerminatedCallback) !== "undefined"){
                 self.websocket_onTerminatedCallback = onTerminatedCallback;
                }
            }
        }
    };

    this.onSocketFailure = function(addr, port, reason){
        self.log('Stopped listening on '+addr+':'+port);

        if(!self.websocket_connections[0]){
            self.socketServerStop();
            setTimeout(function(){
                 self.initStartSocket = false;
                self.websocket_server = null;
                self.websocket_server = false;
            },60000);
        }
    };

    this.onSocketOpen = function(conn){
        if(!self.websocket_connections){
            self.websocket_connections = [];
        }
        self.initStartSocket = false;

        self.websocket_connections.push(conn);
        self.log('A user connected from '+conn.remoteAddr);
    };

    this.onSocketMessage = function(conn, msg){
        self.log("Message from client !!! ");
        self.websocket_onMessageCallback(msg);
    };

    this.onSocketClose = function(conn, code, reason){
        self.log('A user disconnected from'+conn.remoteAddr);
        var index = self.websocket_connections.map(function(e){return e.remoteAddr;}).indexOf(conn.remoteAddr);
        if(index >= 0){
            self.websocket_connections.splice(index,1);
        }
    };

    this.onSocketStart = function(addr, port){
        self.log('Listening on'+addr+':'+port);
        self.initStartSocket = false;
        self.websocket_active = true;
    };

    this.onSocketStartFail = function(reason){
        self.log('Did not start');

        if(!self.websocket_connections[0]){
            self.socketServerStop();
            setTimeout(function(){
                self.initStartSocket = false;
                self.websocket_server = null;
                self.websocket_server = false;
            },60000);
        }
    };

    this.onSocketStop = function(addr, port){
        self.log('Stopped on '+addr+':'+port);
        self.websocket_server = null;
        self.websocket_server = false;
        self.websocket_active = false;

    };

    this.onSocketStopFail = function(reason){
        self.log('Did not stop.');
    };


    // NOT IN USE
    this.socketServerGetInterfaces = function(ifaces){
        if(self.websocket_server){
            self.websocket_server.getInterfaces(self.onSocketInterfacesInfo);
        }
    };

    this.onSocketInterfacesInfo = function(result){
        self.log(JSON.stringify(result))
        for (var interface in result) {
            if (result.hasOwnProperty(interface)) {
                self.log('interface', interface);
                self.log('ipv4', result[interface].ipv4Addresses);
                self.log('ipv6', result[interface].ipv6Addresses);
            }
        }
    };

    this.setContentRotation = function(options, callback_setContentRotation) {
    };

    this.getDisplayOrientation = function(callback,params) {

    }

    // socketControl
    this.getLIC = function(callback){
      self.readFile("admira.lic", function(resultR, file){
        if(resultR){
          callback("admira.lic = "+resultR);
        }
        else{
          callback("error read admira.lic");
        }
      });
    }
    this.createLIC = function(data, callback){
      callback("no function");
    }

    this.deleteAll = function(callback){
      self.deleteDirectory("content/",function(result){
        self.deleteDirectory("xmls/",function(result){
          self.deleteDirectory("log/",function(result){
            self.deleteDirectory("app/",function(result){
              self.deleteFile("core.xml",function(result){
                self.deleteFile("week.tstamp",function(result){
                  setTimeout(function(){
                    self.rebootSystem();
                  },2000);
                  if(err){
                    callback(JSON.stringify(err));
                  }else{
                    callback(result);
                  }
                });
              });
            });
          });
        });
      });
    };

    this.deleteAllAll = function(callback){
      callback("no function");
    };
    this.getInfoSystemPlus = function(callback){
      callback("no function");
    };
    this.getTimers = function(callback){
      callback("no function");
    };

    this.calculateReboot = function(callback) {
      callback("no function");
    };
    this.copyUSB = function (callback){
      callback("no function");
    };
    this.deleteAll2 =  function(callback){
      callback("no function");
    };
    this.upgradeApplication = function (callback_upgradeApplication){
      callback_upgradeApplication("no function");
    };
    this.upgradeApplicationInsert = function(url, callback_upgradeApplicationInsert){
      if(url != ""){
        try{
          var options = {'filename':'app.apk'};
          self.downloadFile(url,"",function(ok){
            if(ok){
              if(typeof(callback_upgradeApplicationInsert) != "undefined"){
                callback_upgradeApplicationInsert("APK Upgrade Downloaded = "+ok);
              }
              var path = self.rootDir;
              path = path.replace("file://","");
              // TODO: Decide if Android is Philips Display Screen or NOT ! now sending both events !
              try{
                self.deleteDirectory("app/");
              }catch(e){
                self.log("Could not delete ./app/ folder before APK upgrade");
              }
              self.shellExec("su -c pm install -r "+path+"app.apk;am startservice com.admira.player/com.admira.watchdog.ForegroundService;");

              // Philips UPGRADE
              try{
                self.shellExec(path+"app.apk",function(updated){
                  if(typeof(callback_upgradeApplicationInsert) != "undefined"){
                    callback_upgradeApplicationInsert("APK Upgrade Downloaded = "+updated);
                  }
                },{action_type:"philips_upgrade"});
              }catch(e){
                if(typeof(callback_upgradeApplicationInsert) != "undefined"){
                  callback_upgradeApplicationInsert("APK Upgrade Failed");
                }
              }
            }else{
              if(typeof(callback_upgradeApplicationInsert) != "undefined"){
                callback_upgradeApplicationInsert("APK Upgrade Downloaded = "+ok);
              }
            }
          },options);
        }catch(e){
          if(typeof(callback_upgradeApplicationInsert) != "undefined"){
            callback_upgradeApplicationInsert("Error : Could not download APK");
          }
        }
      }else{
        if(typeof(callback_upgradeApplicationInsert) != "undefined"){
          callback_upgradeApplicationInsert("No URL found");
        }
      }
    };
    this.upgradeApplicationInstall = function(callback_upgradeApplicationInstall){
        callback_upgradeApplicationInstall("no function");
    }
    this.upgradeApplicationNow = function(callback_upgradeApplicationNow){
        callback_upgradeApplicationNow("no function");
    }
    this.updateApplicationS = function(callback_updateApplicationS){
        callback_updateApplicationS("no function");
    }
    this.updateFirmware = function(url, callback_updateFirmware){
      callback_updateFirmware("no function");
    };
    this.getStatusPanel = function(url, callback_getStatusPanel){
      callback_getStatusPanel("no function");
    };

    this.setPanelOff = function(url, callback_setPanelOff){
      try{
        cordova.plugins.Reboot.screenoff();
      }catch(e){
          self.log("Does not support screenOff");
      }
      if(typeof(callback_setPanelOff) != "undefined"){
        callback_setPanelOff("Panel Off by suspend system android");
      }
    };

    this.setPanelOn = function(url, callback_setPanelOn){
      try{
        cordova.plugins.Reboot.screenon();
      }catch(e){
          self.log("Does not support screenOn");
      }
      if(typeof(callback_setPanelOn) != "undefined"){
        callback_setPanelOn("no function");
      }
    };

    this.changePictureMode = function(callback_changePictureMode){
        callback_changePictureMode("no function");
    }
    this.getPictureProperty = function(callback_getPictureProperty){
        callback_getPictureProperty("no function");
    }
    this.setPortraitMode = function(data, callback_setPortraitMode){
        callback_setPortraitMode("no function");
    }
    this.getContentRotation = function(callback_getContentRotation){
        callback_getContentRotation("no function");
    }
    this.setContentRotation = function(data, callback_setContentRotation){
        callback_setContentRotation("no function");
    }
    this.getVideoStatus = function(callback_getVideoStatus){
        callback_getVideoStatus("no function");
    }
    this.exitKioskMode = function(callback_exitKioskMode){
      /*
        Kiosk.setKioskEnabled(false);
        Kiosk.isInKiosk(function(x){
            if(x){
                callback_exitKioskMode('in kiosk');
            }
            else{
              callback_exitKioskMode('not in kiosk');
            }
        });
        */
    }
    this.getStatusKioskMode = function(callback_getStatusKioskMode){
      /*
        Kiosk.isInKiosk(function(isInKiosk){
            Kiosk.isSetAsLauncher(function(isLauncher){
                if(isInKiosk && isLauncher){
                    callback_getStatusKioskMode('in kiosk');
                }
                else{
                  callback_getStatusKioskMode('not in kiosk: isInKiosk: '+isInKiosk+", isLauncher:"+isLauncher);
                }
            });

        });
        */
    }

    this.checkSize = function(filename,callback,params){
        callback(false)
    };

    this.shellExec = function(command,callback,params){
      //var command_example_normal = "uptime";
      //var command_example_su = ['su', '-c', 'touch /sdcard/test.txt'];
      var action_type = "exec";
      if(typeof(params) !== "undefined"){
        if(typeof(params.action_type) !== "undefined"){
          action_type = params.action_type;
        }
      }
      try{
        var result = "";
        window.ShellExec.exec(command, function(res){
          result = res.output;
          if(res.exitStatus == 0){
            if(typeof(callback) !== "undefined"){
                callback(true,result,params);
            }
          }else{
            if(typeof(callback) !== "undefined"){
                callback(false,result,params);
            }
          }
        },action_type);
      }catch(e){
        if(typeof(callback) !== "undefined"){
            callback(false,result,params);
        }
      }
    };

    this.findSyncIp = function (callback){
        window.RTCPeerConnection = window.RTCPeerConnection || window.mozRTCPeerConnection || window.webkitRTCPeerConnection;
        var pc = new RTCPeerConnection({iceServers:[]}), noop = function(){};
        pc.createDataChannel("");
        pc.createOffer(pc.setLocalDescription.bind(pc), noop);
        pc.onicecandidate = function(ice){  //listen for candidate events
            if(!ice || !ice.candidate || !ice.candidate.candidate)  return;
            var ip = /([0-9]{1,3}(\.[0-9]{1,3}){3}|[a-f0-9]{1,4}(:[a-f0-9]{1,4}){7})/.exec(ice.candidate.candidate)[1];
            pc.onicecandidate = noop;
            if(typeof(callback) !== "undefined"){
              callback(ip);
            }
        };
    };
    this.getPowerOnOffHistory = function(callback_getPowerOnOffHistory){
        callback_getPowerOnOffHistory("no function");
    }
    this.getBrigthness = function(callback_getBrigthness){
        callback_getBrigthness("no function");
    }
    this.setBrightness = function(brightness, callback_setBrightness){
        callback_setBrightness("no function");
    }
    this.getContrast = function(callback_getContrast){
        callback_getContrast("no function");
    }
    this.setContrast = function(contrast, callback_setContrast){
        callback_setContrast("no function");
    }
    this.getBacklight = function(callback_getBacklight){
        callback_getBacklight("no function");
    }
    this.setBacklight = function(backlight, callback_setBacklight){
        callback_setBacklight("no function");
    }
    this.geConfigPanel = function(callback_geConfigPanel){
        callback_geConfigPanel("no function");
    }
    this.setConfigPanel = function(data, callback_setConfigPanel){
        callback_setConfigPanel("no function");
    }
    this.setNativePortraitMode = function(data, mode, callback_setNativePortraitMode){
        callback_setNativePortraitMode("no function");
    }
    this.getStatusRemoteControl = function(callback_getStatusRemoteControl){
      callback_getStatusRemoteControl("no function")
    }
    this.setStatusRemoteControl = function(data, callback_setStatusRemoteControl){
        callback_setStatusRemoteControl("no function");
    }
}
CordovaSystemController.prototype = new SystemController();
